/*
 *          Vanish Firefox Plugin MPL Version 1.1 License
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is "Vanish Firefox Plugin"
 * 
 * The Initial Owner of the Original Code is the University of Washington.
 * The code is Copyright (C) 2009 by University of Washington. All
 * Rights Reserved.
 * 
 * Contributor(s): Amit Levy
 */

/*
   Class: Vanish
   This is the main kernel for Vanish
 */
var Vanish = function() {

	var encText = "Encapsulating Vanish Message...";
	var decText = "Decapsulating Vanish Message...";

	function loadingDiv() {
		var loading = document.createElement("div");
		loading.style.backgroundColor = "#5090FF";
		loading.style.position = "fixed";
		loading.style.left = "0px";
		loading.style.top = "0px";
		loading.style.width = "100%";
		loading.style.height = "100%";
		loading.style.opacity = "0.6";
		return loading;
	}

	function textDiv(text) {
		var div = document.createElement("div");
		div.style.color = "#000";
		div.style.font = "30pt sans-serif";
		div.style.fontWeight = "bold";
		div.style.position = "fixed";
		div.style.width = "100%";
		div.style.textAlign = "center";
		div.appendChild(document.createTextNode(text));
		return div;
	}
	
	function decapsulateError(msg) {
		var str = msg;
		var m = msg.match(/(Got [0-9]+\/[0-9]+ shares)/);
		if (m) {
			str = "Unable to reconstruct your Vanish message (" +
	               m[1] +
	               ").\nThe message must have self-destructed.";
		}
		alert(str);
	}

	function showResult(elm, before, after, isEncapsulated) {
		return ( function(data, time) {
			if (data.decode) {
				data = data.decode();
			}
			var text = before + data + after;
			if (elm) {
				var value = elm.value;
				var startPos = elm.selectionStart;
				var endPos = elm.selectionEnd;
				var chaine = elm.value;
				elm.value = chaine.substring(0, startPos) + text
						+ chaine.substring(endPos, chaine.length);
				elm.selectionStart = startPos;
				elm.selectionEnd = startPos + text.length;
			} else {
				window.openDialog('chrome://vanish/content/showtext.xul',
						'', 'chrome, dialog, resizable=yes', isEncapsulated,
						text, time).focus();
			}
		});

	}

	return {
		prefs : Components.classes["@mozilla.org/preferences-service;1"]
				.getService(Components.interfaces.nsIPrefService).getBranch(
						"extensions.vanish."),

		uuid: function() {
                var id;
                try {
                        id = Vanish.prefs.getCharPref("uuid");
                } catch (e) {
                        var uuidGenerator = Components.classes["@mozilla.org/uuid-generator;1"]
                                               .getService(Components.interfaces.nsIUUIDGenerator);
                        id = uuidGenerator.generateUUID().toString();
                        Vanish.prefs.setCharPref("uuid", id);
                }
                return id;
        },

		/*
		    Function: encapsulate
		    Return an encapsulated VDO.
		
		    Parameters:
		        slient - Set this to true to disable any alert for the user
		        text - The text to sign
		 */
		encapsulate : function(text, callback, callFinal) {
			try {
				var vdoHeader = Vanish.prefs.getCharPref("header");
				var vdoFooter = Vanish.prefs.getCharPref("footer");
				var vdoHeaderText = Vanish.prefs.getCharPref("header_text");

				var params = {};
				params.num_shares = Vanish.prefs.getIntPref("num_shares");
				params.threshold = Vanish.prefs.getIntPref("threshold");
				params.timeout_h = Vanish.prefs.getIntPref("timeout_h");
				var xmlRpc = new XMLRpc(new XMLHttpRequest(), Vanish.prefs
						.getCharPref("rpc_host"), "RemoteVanish.encapsulate", [
						new XMLRpcParam({vid: Vanish.uuid(), ref: content.document.domain}),
						new XMLRpcParam(VBase64.encode(text)),
						new XMLRpcParam(params) ]);
				if (callback) {
					xmlRpc.callAsync( function(response, time) {
						var date = new Date();
						date.setHours(date.getHours() + params.timeout_h + 1);
						response = vdoHeader + "\n" + vdoHeaderText +
								"This message will self destruct by " +
								date.toUTCString() + "\n" + "\n" + response +
								"\n" + vdoFooter;
						callback(response, time);
					}, alert, callFinal);
				} else {
					var response = xmlRpc.call();
					var date = new Date();
					date.setHours(date.getHours() + params.timeout_h + 1);
					response = vdoHeader + "\n" + vdoHeaderText +
							"This messgae will self destruct by " +
							date.toUTCString() + "\n" + "\n" + response +
							"\n" + vdoFooter;
					if (callFinal) {
						callFinal();
					}
					return response;
				}
			} catch (e) {
				throw "Vanish encapsulation failed: " + e;
			}
		},

		encapsulateSelection : function() {
			var bdy = content.document.body;
			loading = loadingDiv();
			encDiv = textDiv(encText);
			bdy.appendChild(loading);
			bdy.appendChild(encDiv);
			try {
				var elm = Selection.isEditable();
				Vanish.encapsulate(Selection.get(), showResult(elm, "", "",
						true), function() {
					bdy.removeChild(encDiv);
					bdy.removeChild(loading);
				});
			} catch (e) {
				alert(e);
			}
		},

		/**
		    Function: decapusate
		    Return the string result from decapsulating a VDO.
		
		    Parameters:
		        slient - Set this to true to disable any alert for the user
		        base64 - The base64 string represtenting the VDO to decapsulate
		 */
		decapsulate : function(base64, callback, callFinal) {
			try {
				var xmlRpc = new XMLRpc(new XMLHttpRequest(), Vanish.prefs
						.getCharPref("rpc_host"), "RemoteVanish.decapsulate",
						[ new XMLRpcParam({vid: Vanish.uuid(), ref: content.document.domain}), new XMLRpcParam(new VBase64(base64)) ]);
				if (callback) {
					xmlRpc.callAsync(callback, decapsulateError, callFinal);
				} else {
					var response = xmlRpc.call();
					if (callFinal) {
						callFinal();
					}
					return response.decode();
				}
			} catch (e) {
				throw "Vanish decapsulation failed: " + e;
			}
		},

		extractVDO : function(initial) {
			var vdoHeader = Vanish.prefs.getCharPref("header");
			var vdoFooter = Vanish.prefs.getCharPref("footer");
			var vdoHeaderText = Vanish.prefs.getCharPref("header_text");
			var vdoRegExp = new RegExp(vdoHeader
					+ "(?:.|\n)*?[^a-zA-Z0-9+=\/]{2}((?:[a-zA-Z0-9+=\/]+\n)+)[^a-zA-Z0-9+=\/]*"
					+ vdoFooter);

			var before = initial.substring(0, initial.indexOf(vdoHeader));
			var footerPos = initial.search(
					/-----END[\s]+VANISH[\s]+MESSAGE-----/)
			var postFooterPos = initial.indexOf('\n', footerPos);
			var after = postFooterPos >= 0 ? initial.substring(postFooterPos, initial.length) : "";
			var result = initial.match(vdoRegExp);
			if (result) {
				return [ before, result[1].replace(/[^a-zA-Z0-9+=\/]/g,''), after ];
			} else {
			}

			return false;
		},

		decapsulateSelection : function() {
			var bdy = content.document.body;
			try {
				var elm = Selection.isEditable();
				var data = Selection.get();
				var vdo = Vanish.extractVDO(data);
				if (vdo) {
					loading = loadingDiv();
					decDiv = textDiv(decText);
					bdy.appendChild(loading);
					bdy.appendChild(decDiv);
					Vanish.decapsulate(vdo[1], showResult(elm, vdo[0], vdo[2],
							false), function() {
						bdy.removeChild(decDiv);
						bdy.removeChild(loading);
					});
				} else {
					alert("You did not select a Vanish message. Make sure you've " +
							"highlighted at least the entire message which will have " +
							"a format similar to:\n\n" +
							"-----BEGIN VANISH MESSAGE-----\n" +
							"...\n...\n" +
							"-----END VANISH MESSAGE-----\n");
				}
			} catch (e) {
				alert(e);
			}
		}
	};

}();
